@@ -8,8 +8,6 @@
 #include <QDesktopServices>
 #include <QFile>
 #include <QGeoCoordinate>
-#include <QThread>
-#include <QWaitCondition>
 // QXmpp
 #include "QXmppSasl2UserAgent.h"
 #include <QXmppUri.h>
@@ -18,12 +16,13 @@
 #include "AccountDb.h"
 #include "AtmController.h"
 #include "Blocking.h"
-#include "ChatController.h"
+#include "CallController.h"
+#include "ChatStateCache.h"
 #include "CredentialsValidator.h"
 #include "DiscoveryController.h"
 #include "EncryptionController.h"
 #include "FileSharingController.h"
-#include "FutureUtils.h"
+#include "Globals.h"
 #include "GroupChatController.h"
 #include "LogHandler.h"
 #include "MessageController.h"
@@ -39,63 +38,6 @@
 
 constexpr QStringView GEO_LOCATION_WEB_URL = u"https://osmand.net/map/?pin=%1,%2#16/%1/%2";
 
-/**
- * Allows to run a function and waits for the thread to be started before returning to the calling
- * thread.
- *
- * This is needed because QXmppMigrationManager expects the classes of the import/export functions
- * to be registered while they are running in the right/final thread.
- */
-class XmppThread : public QThread
-{
-public:
-    explicit XmppThread(std::future<void> &&future)
-        : m_future(std::move(future))
-    {
-    }
-
-    ~XmppThread() override
-    {
-        requestInterruption();
-        quit();
-        wait();
-    }
-
-    void waitForStarted(QThread::Priority priority = InheritPriority)
-    {
-        if (isRunning()) {
-            return;
-        }
-
-        QMutexLocker locker(&m_mutex);
-        QThread::start(priority);
-        m_condition.wait(&m_mutex);
-    }
-
-    template<typename Function, typename... Args>
-    static XmppThread *create(Function &&f, Args &&...args)
-    {
-        using DecayedFunction = typename std::decay<Function>::type;
-        auto threadFunction = [f = static_cast<DecayedFunction>(std::forward<Function>(f))](auto &&...largs) mutable -> void {
-            (void)std::invoke(std::move(f), std::forward<decltype(largs)>(largs)...);
-        };
-        return new XmppThread(std::move(std::async(std::launch::deferred, std::move(threadFunction), std::forward<Args>(args)...)));
-    }
-
-protected:
-    void run() override
-    {
-        m_future.get();
-        m_condition.wakeOne();
-        QThread::exec();
-    }
-
-private:
-    QWaitCondition m_condition;
-    QMutex m_mutex;
-    std::future<void> m_future;
-};
-
 AccountSettings::AccountSettings(Data data, QObject *parent)
     : QObject(parent)
     , m_data(data)
@@ -459,26 +401,66 @@ void AccountSettings::generateUserAgentDeviceId()
     }
 }
 
-Connection::Connection(ClientWorker *clientWorker, QObject *parent)
+Connection::Connection(ClientController *clientController, QObject *parent)
     : QObject(parent)
-    , m_clientWorker(clientWorker)
+    , m_clientController(clientController)
 {
-    connect(m_clientWorker, &ClientWorker::connectionStateChanged, this, &Connection::setState);
-    connect(m_clientWorker, &ClientWorker::connectionErrorChanged, this, &Connection::setError);
+    connect(m_clientController, &ClientController::connectionStateChanged, this, &Connection::setState);
+    connect(m_clientController, &ClientController::connectionErrorChanged, this, &Connection::setError);
 }
 
 void Connection::logIn()
 {
-    runOnThread(m_clientWorker, [this]() {
-        m_clientWorker->logIn();
-    });
+    m_clientController->logIn();
 }
 
 void Connection::logOut(bool isApplicationBeingClosed)
 {
-    runOnThread(m_clientWorker, [this, isApplicationBeingClosed]() {
-        m_clientWorker->logOut(isApplicationBeingClosed);
-    });
+    QList<QFuture<void>> futures;
+
+    for (auto &logoutTaskWrapper : m_logoutTaskWrappers) {
+        auto promise = logoutTaskWrapper.promise;
+        futures.append(promise->future());
+        promise->start();
+    }
+
+    if (futures.isEmpty()) {
+        m_clientController->logOut(isApplicationBeingClosed);
+    } else {
+        joinVoidFutures(std::move(futures)).then(this, [this, isApplicationBeingClosed]() {
+            m_logoutTaskWrappers.clear();
+            m_clientController->logOut(isApplicationBeingClosed);
+        });
+    }
+}
+
+std::shared_ptr<QPromise<void>> Connection::addLogoutTask(std::function<void()> logoutTask)
+{
+    auto promise = std::make_shared<QPromise<void>>();
+
+    auto watcher = std::make_shared<QFutureWatcher<void>>();
+    watcher->setFuture(promise->future());
+
+    connect(watcher.get(), &QFutureWatcher<void>::started, this, logoutTask);
+
+    LogoutTaskWrapper logoutTaskWrapper = {
+        .promise = promise,
+        .watcher = watcher,
+    };
+
+    m_logoutTaskWrappers.append(logoutTaskWrapper);
+
+    return promise;
+}
+
+void Connection::removeLogoutTask(std::shared_ptr<QPromise<void>> promise)
+{
+    m_logoutTaskWrappers.erase(std::ranges::remove_if(m_logoutTaskWrappers,
+                                                      [promise](LogoutTaskWrapper &logoutTaskWrapper) {
+                                                          return logoutTaskWrapper.promise == promise;
+                                                      })
+                                   .begin(),
+                               m_logoutTaskWrappers.end());
 }
 
 Enums::ConnectionState Connection::state() const
@@ -489,12 +471,12 @@ Enums::ConnectionState Connection::state() const
 QString Connection::stateText() const
 {
     switch (m_state) {
-    case Enums::ConnectionState::StateConnected:
-        return tr("Connected");
-    case Enums::ConnectionState::StateConnecting:
-        return tr("Connecting…");
     case Enums::ConnectionState::StateDisconnected:
         return tr("Disconnected");
+    case Enums::ConnectionState::StateConnecting:
+        return tr("Connecting…");
+    case Enums::ConnectionState::StateConnected:
+        return tr("Connected");
     }
     return {};
 }
@@ -502,37 +484,37 @@ QString Connection::stateText() const
 QString Connection::errorText() const
 {
     switch (m_error) {
-    case ClientWorker::NoError:
+    case ClientController::NoError:
         return {};
-    case ClientWorker::AuthenticationFailed:
+    case ClientController::AuthenticationFailed:
         return tr("Invalid username or password.");
-    case ClientWorker::NotConnected:
+    case ClientController::NotConnected:
         return tr("Cannot connect to the server. Please check your internet connection.");
-    case ClientWorker::TlsFailed:
+    case ClientController::TlsFailed:
         return tr("Error while trying to connect securely.");
-    case ClientWorker::TlsNotAvailable:
+    case ClientController::TlsNotAvailable:
         return tr("The server doesn't support secure connections.");
-    case ClientWorker::DnsError:
+    case ClientController::DnsError:
         return tr("Could not connect to the server. Please check your internet connection or your server name.");
-    case ClientWorker::ConnectionRefused:
+    case ClientController::ConnectionRefused:
         return tr("The server is offline or blocked by a firewall.");
-    case ClientWorker::NoSupportedAuth:
+    case ClientController::NoSupportedAuth:
         return tr("Authentication protocol not supported by the server.");
-    case ClientWorker::KeepAliveError:
+    case ClientController::KeepAliveError:
         return tr("The connection could not be refreshed.");
-    case ClientWorker::NoNetworkPermission:
+    case ClientController::NoNetworkPermission:
         return tr("The internet access is not permitted. Please check your system's internet access configuration.");
-    case ClientWorker::RegistrationUnsupported:
+    case ClientController::RegistrationUnsupported:
         return tr("This server does not support registration.");
-    case ClientWorker::EmailConfirmationRequired:
+    case ClientController::EmailConfirmationRequired:
         return tr("Could not log in. Confirm the email message you received first.");
-    case ClientWorker::UnknownError:
+    case ClientController::UnknownError:
         return tr("Could not connect to the server.");
     }
     Q_UNREACHABLE();
 }
 
-ClientWorker::ConnectionError Connection::error() const
+ClientController::ConnectionError Connection::error() const
 {
     return m_error;
 }
@@ -545,11 +527,13 @@ void Connection::setState(Enums::ConnectionState state)
 
         if (state == Enums::ConnectionState::StateConnected) {
             Q_EMIT connected();
+        } else if (state == Enums::ConnectionState::StateDisconnected) {
+            Q_EMIT disconnected();
         }
     }
 }
 
-void Connection::setError(ClientWorker::ConnectionError error)
+void Connection::setError(ClientController::ConnectionError error)
 {
     // For displaying errors to the user, every new error (even if it is the same as before) must be
     // emitted.
@@ -566,54 +550,37 @@ Account::Account(QObject *parent)
 Account::Account(AccountSettings::Data accountSettingsData, QObject *parent)
     : QObject(parent)
     , m_settings(new AccountSettings(accountSettingsData, this))
-    , m_client([this] {
-        ClientWorker *worker = nullptr;
-        auto thread = XmppThread::create([&, this] {
-            worker = new ClientWorker(m_settings);
-        });
-
-        thread->setObjectName(u"XMPP");
-        thread->waitForStarted();
-
-        new LogHandler(m_settings, worker->xmppClient(), this);
-
-        return XmppClient{thread, worker};
-    }())
-    , m_connection(new Connection(m_client.worker, this))
+    , m_clientController(new ClientController(m_settings))
+    , m_connection(new Connection(m_clientController, this))
     , m_presenceCache(new PresenceCache(this))
-    , m_atmController(new AtmController(m_client.worker->atmManager(), this))
-    , m_blockingController(new BlockingController(m_settings, m_connection, m_client.worker, this))
-    , m_encryptionController(new EncryptionController(m_settings, m_presenceCache, m_client.worker->omemoManager(), this))
-    , m_rosterController(new RosterController(m_settings, m_connection, m_encryptionController, m_client.worker->rosterManager(), this))
-    , m_fileSharingController(new FileSharingController(m_settings, m_connection, m_client.worker, this))
+    , m_atmController(new AtmController(m_clientController->atmManager(), this))
+    , m_blockingController(new BlockingController(m_settings, m_connection, m_clientController, this))
+    , m_encryptionController(new EncryptionController(m_settings, m_presenceCache, m_clientController->omemoManager(), this))
+    , m_rosterController(new RosterController(m_settings, m_connection, m_encryptionController, m_clientController->rosterManager(), this))
+    , m_fileSharingController(new FileSharingController(m_settings, m_connection, m_clientController, this))
     , m_messageController(new MessageController(m_settings,
                                                 m_connection,
                                                 m_encryptionController,
                                                 m_rosterController,
                                                 m_fileSharingController,
-                                                m_client.worker->xmppClient(),
-                                                m_client.worker->mamManager(),
-                                                m_client.worker->messageReceiptManager(),
+                                                m_clientController->xmppClient(),
+                                                m_clientController->mamManager(),
+                                                m_clientController->messageReceiptManager(),
                                                 this))
-    , m_groupChatController(new GroupChatController(m_settings, m_messageController, m_client.worker->mixManager(), this))
+    , m_chatStateCache(new ChatStateCache(m_connection, m_messageController))
+    , m_groupChatController(new GroupChatController(m_settings, m_messageController, m_clientController->mixManager(), this))
     , m_notificationController(new NotificationController(m_settings, m_messageController, this))
-    , m_vCardController(new VCardController(m_settings, m_connection, m_presenceCache, m_client.worker->xmppClient(), m_client.worker->vCardManager(), this))
-    , m_registrationController(new RegistrationController(m_settings, m_connection, m_encryptionController, m_vCardController, m_client.worker, this))
-    , m_versionController(new VersionController(m_presenceCache, m_client.worker->versionManager()))
+    , m_callController(
+          new CallController(m_settings, m_connection, m_notificationController, m_clientController->jmiManager(), m_clientController->callManager(), this))
+    , m_vCardController(
+          new VCardController(m_settings, m_connection, m_presenceCache, m_clientController->xmppClient(), m_clientController->vCardManager(), this))
+    , m_registrationController(new RegistrationController(m_settings, m_connection, m_encryptionController, m_vCardController, m_clientController, this))
+    , m_versionController(new VersionController(m_presenceCache, m_clientController->versionManager()))
 {
-    new DiscoveryController(m_settings, m_connection, m_client.worker->discoveryManager(), this);
+    new LogHandler(m_settings, m_clientController->xmppClient(), this);
+    new DiscoveryController(m_settings, m_connection, m_clientController->discoveryManager(), this);
 
-    runOnThread(m_client.worker, [this]() {
-        m_client.worker->initialize(m_atmController, m_encryptionController, m_messageController, m_registrationController, m_presenceCache);
-    });
-}
-
-Account::~Account()
-{
-    m_client.worker->deleteLater();
-
-    m_client.thread->quit();
-    m_client.thread->wait();
+    m_clientController->initialize(m_atmController, m_encryptionController, m_messageController, m_registrationController, m_presenceCache);
 }
 
 AccountSettings *Account::settings() const
@@ -626,9 +593,9 @@ Connection *Account::connection() const
     return m_connection;
 }
 
-ClientWorker *Account::clientWorker() const
+ClientController *Account::clientController() const
 {
-    return m_client.worker;
+    return m_clientController;
 }
 
 AtmController *Account::atmController() const
@@ -641,6 +608,11 @@ BlockingController *Account::blockingController() const
     return m_blockingController;
 }
 
+CallController *Account::callController() const
+{
+    return m_callController;
+}
+
 EncryptionController *Account::encryptionController() const
 {
     return m_encryptionController;
@@ -686,6 +658,11 @@ VersionController *Account::versionController() const
     return m_versionController;
 }
 
+ChatStateCache *Account::chatStateCache() const
+{
+    return m_chatStateCache;
+}
+
 PresenceCache *Account::presenceCache() const
 {
     return m_presenceCache;
