@@ -20,7 +20,6 @@
 #include "AccountDb.h"
 #include "Algorithms.h"
 #include "Encryption.h"
-#include "FutureUtils.h"
 #include "KaidanCoreLog.h"
 #include "MainController.h"
 #include "RosterDb.h"
@@ -317,7 +316,7 @@ QFuture<bool> AccountMigrationController::startMigration(Account *oldAccount)
     promise->start();
 
     m_oldAccount = oldAccount;
-    auto *migrationManager = m_oldAccount->clientWorker()->accountMigrationManager();
+    auto *migrationManager = m_oldAccount->clientController()->accountMigrationManager();
 
     connect(migrationManager, &QXmppAccountMigrationManager::errorOccurred, this, [this, promise](const QXmppError &error) mutable {
         informUser(error.description);
@@ -333,22 +332,16 @@ QFuture<bool> AccountMigrationController::startMigration(Account *oldAccount)
                 reportFinishedResult(*promise, false);
             }
         } else {
-            callRemoteTask(
-                migrationManager,
-                [this, migrationManager]() {
-                    return std::pair{migrationManager->exportData(), this};
-                },
-                this,
-                [this, promise](QXmppAccountMigrationManager::Result<QXmppExportData> &&result) mutable {
-                    if (const auto error = std::get_if<QXmppError>(&result)) {
-                        informUser(error->description);
-                        reportFinishedResult(*promise, false);
-                    } else {
-                        m_exportData = std::move(std::get<QXmppExportData>(std::move(result)));
-                        m_exportData.setExtension(exportClientSettings());
-                        reportFinishedResult(*promise, true);
-                    }
-                });
+            migrationManager->exportData().then(this, [this, promise](QXmppAccountMigrationManager::Result<QXmppExportData> &&result) mutable {
+                if (const auto error = std::get_if<QXmppError>(&result)) {
+                    informUser(error->description);
+                    reportFinishedResult(*promise, false);
+                } else {
+                    m_exportData = std::move(std::get<QXmppExportData>(std::move(result)));
+                    m_exportData.setExtension(exportClientSettings());
+                    reportFinishedResult(*promise, true);
+                }
+            });
         }
 
     } else {
@@ -364,41 +357,29 @@ QFuture<void> AccountMigrationController::finalizeMigration(Account *newAccount)
     auto promise = std::make_shared<QPromise<void>>();
     promise->start();
 
-    auto *migrationManager = newAccount->clientWorker()->accountMigrationManager();
+    newAccount->clientController()->accountMigrationManager()->importData(m_exportData).then(this, [this, promise, newAccount](auto &&result) mutable {
+        if (const auto accountFilePath = diskAccountFilePath(); const auto error = std::get_if<QXmppError>(&result)) {
+            saveAccountDataToDisk(accountFilePath, m_exportData);
+            informUser(error->description);
+            promise->finish();
+        } else {
+            if (QFile::exists(accountFilePath)) {
+                QFile::remove(accountFilePath);
+            }
 
-    callRemoteTask(
-        migrationManager,
-        [this, migrationManager, exportData = m_exportData]() {
-            return std::pair{migrationManager->importData(exportData), this};
-        },
-        this,
-        [this, promise, newAccount](auto &&result) mutable {
-            if (const auto accountFilePath = diskAccountFilePath(); const auto error = std::get_if<QXmppError>(&result)) {
-                saveAccountDataToDisk(accountFilePath, m_exportData);
-                informUser(error->description);
-                promise->finish();
-            } else {
-                if (QFile::exists(accountFilePath)) {
-                    QFile::remove(accountFilePath);
-                }
-
-                if (const auto oldClientSettings = m_exportData.extension<ClientSettings>(); oldClientSettings) {
-                    importClientSettings(newAccount, *oldClientSettings)
-                        .then(this, [this, promise, newAccount, oldContacts = oldClientSettings->rosterContacts()]() mutable {
-                            auto *movedManager = m_oldAccount->clientWorker()->movedManager();
-
-                            callRemoteTask(
-                                movedManager,
-                                [this, movedManager, newAccountJid = newAccount->settings()->jid()]() {
-                                    return std::pair{movedManager->publishStatement(newAccountJid), this};
-                                },
-                                this,
-                                [this, promise, newAccount, oldContacts](auto &&result) mutable {
-                                    if (const auto error = std::get_if<QXmppError>(&result)) {
-                                        informUser(error->description);
-                                        promise->finish();
-                                    } else {
-                                        notifyContacts(newAccount, oldContacts).then(this, [this, promise, newAccount](auto &&result) mutable {
+            if (const auto oldClientSettings = m_exportData.extension<ClientSettings>(); oldClientSettings) {
+                importClientSettings(newAccount, *oldClientSettings)
+                    .then([this, promise, newAccount, oldContacts = oldClientSettings->rosterContacts()]() mutable {
+                        m_oldAccount->clientController()
+                            ->movedManager()
+                            ->publishStatement(newAccount->settings()->jid())
+                            .then(this, [this, promise, newAccount, oldContacts](auto &&result) mutable {
+                                if (const auto error = std::get_if<QXmppError>(&result)) {
+                                    informUser(error->description);
+                                    promise->finish();
+                                } else {
+                                    notifyContacts(newAccount, oldContacts)
+                                        .then([this, promise, newAccount](QXmppAccountMigrationManager::Result<> result) mutable {
                                             if (const auto error = std::get_if<QXmppError>(&result)) {
                                                 informUser(error->description);
                                             } else {
@@ -409,14 +390,14 @@ QFuture<void> AccountMigrationController::finalizeMigration(Account *newAccount)
 
                                             promise->finish();
                                         });
-                                    }
-                                });
-                        });
-                } else {
-                    promise->finish();
-                }
+                                }
+                            });
+                    });
+            } else {
+                promise->finish();
             }
-        });
+        }
+    });
 
     return promise->future();
 }
@@ -594,7 +575,7 @@ QFuture<void> AccountMigrationController::importClientSettings(Account *newAccou
                 }
             }
 
-            joinVoidFutures(this, std::move(futures)).then([promise]() mutable {
+            joinVoidFutures(std::move(futures)).then([promise]() mutable {
                 promise->finish();
             });
         });
@@ -602,41 +583,40 @@ QFuture<void> AccountMigrationController::importClientSettings(Account *newAccou
     return promise->future();
 }
 
-QXmppTask<QXmppAccountMigrationManager::Result<>> AccountMigrationController::notifyContacts(Account *newAccount, const QList<QString> &contactJids)
+QFuture<QXmppAccountMigrationManager::Result<>> AccountMigrationController::notifyContacts(Account *newAccount, const QList<QString> &contactJids)
 {
     if (contactJids.isEmpty()) {
-        return makeReadyTask<QXmppAccountMigrationManager::Result<>>({});
+        return QtFuture::makeReadyValueFuture<QXmppAccountMigrationManager::Result<>>({});
     }
 
-    QXmppPromise<QXmppAccountMigrationManager::Result<>> promise;
+    auto promise = std::make_shared<QPromise<QXmppAccountMigrationManager::Result<>>>();
+    promise->start();
+
     auto counter = std::make_shared<int>(contactJids.size());
 
     for (const QString &contactJid : contactJids) {
-        auto *movedManager = newAccount->clientWorker()->movedManager();
-
-        callRemoteTask(
-            movedManager,
-            [this, movedManager, contactJid, oldAccountJid = m_exportData.accountJid()]() {
-                return std::pair{movedManager->notifyContact(contactJid, oldAccountJid, false), this};
-            },
-            this,
-            [promise, counter](auto &&result) mutable {
-                if (promise.task().isFinished()) {
+        newAccount->clientController()
+            ->movedManager()
+            ->notifyContact(contactJid, m_exportData.accountJid(), false)
+            .then(this, [promise, counter](auto &&result) mutable {
+                if (promise->future().isFinished()) {
                     return;
                 }
 
                 if (const auto error = std::get_if<QXmppError>(&result)) {
-                    promise.finish(*error);
+                    promise->addResult(*error);
+                    promise->finish();
                     return;
                 }
 
                 if ((--(*counter)) == 0) {
-                    promise.finish(QXmpp::Success());
+                    promise->addResult(QXmpp::Success());
+                    promise->finish();
                 }
             });
     }
 
-    return promise.task();
+    return promise->future();
 }
 
 #include "moc_AccountMigrationController.cpp"
