@@ -63,7 +63,7 @@ void AuthenticatableEncryptionKeyModel::setChatJid(const QString &chatJid)
 
 bool AuthenticatableEncryptionKeyModel::contains(const QString &keyId)
 {
-    return std::any_of(m_keys.cbegin(), m_keys.cend(), [keyId](const Key &key) {
+    return std::ranges::any_of(m_keys, [keyId](const Key &key) {
         return key.id == keyId;
     });
 }
@@ -91,7 +91,7 @@ void AuthenticatableEncryptionKeyModel::handleDevicesChanged(QList<QString> jids
 
 void AuthenticatableEncryptionKeyModel::updateKeys()
 {
-    encryptionController()->devices({m_chatJid}).then(this, [this](QList<EncryptionController::Device> &&devices) {
+    encryptionController()->devices({m_chatJid}).then([this](QList<EncryptionController::Device> &&devices) {
         beginResetModel();
 
         m_keys = transformFilter<QList<Key>>(devices, [](const EncryptionController::Device &device) -> std::optional<Key> {
