@@ -36,9 +36,9 @@ QVariant AuthenticatedEncryptionKeyModel::data(const QModelIndex &index, int rol
 
     Key key;
 
-    if (row == 0) {
-        key.deviceLabel = m_ownKey.deviceLabel;
-        key.id = m_ownKey.id;
+    if (m_ownKey && row == 0) {
+        key.deviceLabel = m_ownKey->deviceLabel;
+        key.id = m_ownKey->id;
     } else {
         key = m_keys.at(row - 1);
     }
@@ -72,23 +72,33 @@ void AuthenticatedEncryptionKeyModel::handleDevicesChanged(const QList<QString>
 
 void AuthenticatedEncryptionKeyModel::updateOwnKey()
 {
-    encryptionController()->ownDevice().then(this, [this](EncryptionController::OwnDevice &&ownDevice) {
-        beginInsertRows(QModelIndex(), 0, 0);
+    const auto ownDevice = encryptionController()->ownDevice();
+
+    auto setOwnKey = [this, ownDevice]() {
         m_ownKey = {ownDevice.label + QStringLiteral(" · ") + tr("This device"), ownDevice.keyId};
+    };
+
+    if (m_ownKey) {
+        setOwnKey();
+        const auto modelIndex = index(0);
+        dataChanged(modelIndex, modelIndex);
+    } else {
+        beginInsertRows(QModelIndex(), 0, 0);
+        setOwnKey();
         endInsertRows();
-    });
+    }
 }
 
 void AuthenticatedEncryptionKeyModel::updateKeys()
 {
-    encryptionController()->keys({accountJid()}, QXmpp::TrustLevel::Authenticated).then(this, [this](QHash<QString, QHash<QString, QXmpp::TrustLevel>> &&keys) {
+    encryptionController()->keys({accountJid()}, QXmpp::TrustLevel::Authenticated).then([this](QHash<QString, QHash<QString, QXmpp::TrustLevel>> &&keys) {
         const auto keyIds = keys.value(accountJid()).keys();
 
         auto authenticatedKeys = transform(keyIds, [](const QString &keyId) {
             return Key{{}, keyId};
         });
 
-        encryptionController()->devices({accountJid()}).then(this, [this, authenticatedKeys](QList<EncryptionController::Device> &&devices) mutable {
+        encryptionController()->devices({accountJid()}).then([this, authenticatedKeys](QList<EncryptionController::Device> &&devices) mutable {
             for (const auto &device : std::as_const(devices)) {
                 for (auto &authenticatedKey : authenticatedKeys) {
                     if (device.keyId == authenticatedKey.id) {
@@ -97,9 +107,29 @@ void AuthenticatedEncryptionKeyModel::updateKeys()
                 }
             }
 
-            beginResetModel();
-            m_keys = authenticatedKeys;
-            endResetModel();
+            auto setKeys = [this, authenticatedKeys]() {
+                m_keys = authenticatedKeys;
+            };
+
+            if (m_ownKey) {
+                if (m_keys.isEmpty()) {
+                    beginInsertRows(QModelIndex(), 1, authenticatedKeys.size());
+                    setKeys();
+                    endInsertRows();
+                } else {
+                    setKeys();
+                    dataChanged(index(1), index(m_keys.size()));
+                }
+            } else {
+                if (m_keys.isEmpty()) {
+                    beginInsertRows(QModelIndex(), 0, authenticatedKeys.size() - 1);
+                    setKeys();
+                    endInsertRows();
+                } else {
+                    setKeys();
+                    dataChanged(index(0), index(m_keys.size() - 1));
+                }
+            }
         });
     });
 }
