@@ -90,15 +90,11 @@ void ChatHintModel::handleButtonClicked(int i, ChatHintButton::Type type)
     switch (type) {
     case ChatHintButton::Dismiss:
         if (i == chatHintIndex(ChatHintButton::AllowPresenceSubscription)) {
-            const auto jid = m_chatController->jid();
-
-            m_rosterController->refuseSubscriptionToPresence(jid).then(this, [this, i, jid](bool succeeded) {
-                if (succeeded) {
-                    removeChatHint(i);
-                } else {
-                    setLoading(i, false);
-                }
-            });
+            if (m_rosterController->refuseSubscriptionToPresence(m_chatController->jid())) {
+                removeChatHint(i);
+            } else {
+                setLoading(i, false);
+            }
         } else {
             removeChatHint(i);
         }
@@ -113,15 +109,12 @@ void ChatHintModel::handleButtonClicked(int i, ChatHintButton::Type type)
         return;
     case ChatHintButton::AllowPresenceSubscription: {
         setLoading(i, true);
-        const auto jid = m_chatController->jid();
 
-        m_rosterController->acceptSubscriptionToPresence(jid).then(this, [this, i, jid](bool succeeded) {
-            if (succeeded) {
-                removeChatHint(i);
-            } else {
-                setLoading(i, false);
-            }
-        });
+        if (m_rosterController->acceptSubscriptionToPresence(m_chatController->jid())) {
+            removeChatHint(i);
+        } else {
+            setLoading(i, false);
+        }
 
         return;
     }
@@ -164,7 +157,7 @@ void ChatHintModel::handleConnectionStateChanged()
 
 void ChatHintModel::handleConnectionErrorChanged()
 {
-    const auto chatHintShouldBeShown = m_connection->error() != ClientWorker::NoError;
+    const auto chatHintShouldBeShown = m_connection->error() != ClientController::NoError;
 
     if (const auto i = chatHintIndex(ChatHintButton::ShowConnectionError); i == -1) {
         if (chatHintShouldBeShown) {
@@ -253,7 +246,7 @@ void ChatHintModel::handleGroupChatDeleted(const QString &groupChatJid)
 int ChatHintModel::addEnableAccountChatHint()
 {
     return addChatHint({
-        tr("Your account is not enabled - Messages are sent and received once enabled"),
+        tr("Your account is disabled - Messages are exchanged once connected"),
         {ChatHintButton{ChatHintButton::Dismiss, tr("Dismiss")}, ChatHintButton{ChatHintButton::EnableAccount, tr("Enable")}},
         false,
         tr("Enabling…"),
@@ -414,7 +407,7 @@ bool ChatHintModel::hasButton(int i, ChatHintButton::Type buttonType) const
 {
     const auto buttons = m_chatHints.at(i).buttons;
 
-    return std::any_of(buttons.cbegin(), buttons.cend(), [buttonType](const ChatHintButton &button) {
+    return std::ranges::any_of(buttons, [buttonType](const ChatHintButton &button) {
         return button.type == buttonType;
     });
 }
