@@ -10,7 +10,7 @@
 // QXmpp
 #include <QXmppBitsOfBinaryContentId.h>
 // Kaidan
-#include "Globals.h"
+#include "ImageProvider.h"
 
 DataFormModel::DataFormModel(QObject *parent)
     : QAbstractListModel(parent)
@@ -68,7 +68,8 @@ bool DataFormModel::setData(const QModelIndex &index, const QVariant &value, int
     if (!index.isValid() || !hasIndex(index.row(), index.column(), index.parent()))
         return false;
 
-    QXmppDataForm::Field field = m_form.fields().at(index.row());
+    auto fields = m_form.fields();
+    auto &field = fields[index.row()];
 
     switch (role) {
     case Key:
@@ -93,7 +94,7 @@ bool DataFormModel::setData(const QModelIndex &index, const QVariant &value, int
         return false;
     }
 
-    m_form.fields()[index.row()] = field;
+    m_form.setFields(fields);
     Q_EMIT dataChanged(index, index, QList<int>() << role);
     return true;
 }
@@ -128,17 +129,18 @@ QString DataFormModel::instructions() const
     return m_form.instructions();
 }
 
-QString DataFormModel::mediaSourceUri(const QXmppDataForm::Field &field) const
+QUrl DataFormModel::mediaSourceUri(const QXmppDataForm::Field &field) const
 {
-    QString mediaSourceUri;
+    QUrl mediaSourceUri;
     const auto mediaSources = field.mediaSources();
 
     for (const auto &mediaSource : mediaSources) {
-        mediaSourceUri = mediaSource.uri().toString();
+        mediaSourceUri = mediaSource.uri();
         // Prefer Bits of Binary URIs.
         // In most cases, the data has been received already then.
-        if (QXmppBitsOfBinaryContentId::isBitsOfBinaryContentId(mediaSourceUri, true))
-            return QStringLiteral("image://%1/%2").arg(QStringLiteral(BITS_OF_BINARY_IMAGE_PROVIDER_NAME), mediaSourceUri);
+        if (QXmppBitsOfBinaryContentId::isBitsOfBinaryContentId(mediaSourceUri.toString(), true)) {
+            return ImageProvider::generatedBitsOfBinaryImageUrl(mediaSourceUri);
+        }
     }
 
     return mediaSourceUri;
