@@ -5,16 +5,15 @@
 #include "DatabaseComponent.h"
 
 // Qt
-#include <QSqlDatabase>
 #include <QSqlDriver>
 #include <QSqlQuery>
 #include <QSqlRecord>
 // Kaidan
 #include "Database.h"
 
-DatabaseComponent::DatabaseComponent(Database *database, QObject *parent)
+DatabaseComponent::DatabaseComponent(QObject *parent)
     : QObject(parent)
-    , m_database(database)
+    , m_database(Database::instance())
 {
 }
 
