@@ -5,8 +5,6 @@
 
 #pragma once
 
-// std
-#include <mutex>
 // Qt
 #include <QObject>
 // Kaidan
@@ -71,18 +69,16 @@ private:
 class FileProgressCache
 {
 public:
-    ~FileProgressCache();
-
     static FileProgressCache &instance();
 
     std::optional<FileProgress> progress(qint64 fileId);
-    void reportProgress(qint64 fileId, std::optional<FileProgress> progress);
+    void reportProgress(qint64 fileId, const FileProgress &progress);
+    void reportFinished(qint64 fileId);
 
     void cancelTransfers(const QString &jid);
 
 private:
     FileProgressCache();
 
-    std::mutex m_mutex;
     std::unordered_map<qint64, FileProgress> m_files;
 };
