@@ -11,46 +11,47 @@
 // QXmpp
 #include <QXmppClient.h>
 // Kaidan
-#include "KaidanCoreLog.h"
+#include "Account.h"
+#include "KaidanXmppLog.h"
 
-LogHandler::LogHandler(QXmppClient *client, bool enable, QObject *parent)
+LogHandler::LogHandler(AccountSettings *accountSettings, QXmppClient *client, QObject *parent)
     : QObject(parent)
-    , m_client(client)
+    , m_accountSettings(accountSettings)
 {
-    client->logger()->setLoggingType(QXmppLogger::SignalLogging);
-    enableLogging(enable);
-}
-
-void LogHandler::enableLogging(bool enabled)
-{
-    // check if we need to change something
-    if (this->enabled == enabled)
-        return;
-    // update enabled status
-    this->enabled = enabled;
-
-    // apply change: enable or disable
-    if (enabled)
-        connect(m_client->logger(), &QXmppLogger::message, this, &LogHandler::handleLog);
-    else
-        disconnect(m_client->logger(), &QXmppLogger::message, this, &LogHandler::handleLog);
+    auto *logger = new QXmppLogger(client);
+    client->setLogger(logger);
+    logger->setLoggingType(QXmppLogger::SignalLogging);
+    connect(logger, &QXmppLogger::message, this, &LogHandler::handleLog);
 }
 
 void LogHandler::handleLog(QXmppLogger::MessageType type, const QString &text)
 {
+    // Filter out stream management acknowledgments.
+    if ((type == QXmppLogger::ReceivedMessage || type == QXmppLogger::SentMessage)
+        && (text.startsWith(u"<a xmlns=\"urn:xmpp:sm:3\"") || text.startsWith(u"<a xmlns='urn:xmpp:sm:3'") || text.startsWith(u"<r xmlns=\"urn:xmpp:sm:3\"")
+            || text.startsWith(u"<r xmlns='urn:xmpp:sm:3'"))) {
+        return;
+    }
+
     switch (type) {
+    case QXmppLogger::DebugMessage:
+        qCDebug(KAIDAN_XMPP_LOG).noquote() << m_accountSettings->jid() << "[client] [debug]" << text;
+        break;
     case QXmppLogger::ReceivedMessage:
-        qCDebug(KAIDAN_CORE_LOG).noquote() << "[incoming] <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<" << Qt::endl
+        qCDebug(KAIDAN_XMPP_LOG).noquote() << m_accountSettings->jid() << "[incoming] <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<"
+                                           << Qt::endl
                                            << makeXmlPretty(text);
         break;
     case QXmppLogger::SentMessage:
-        qCDebug(KAIDAN_CORE_LOG).noquote() << "[outgoing] >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>" << Qt::endl
+        qCDebug(KAIDAN_XMPP_LOG).noquote() << m_accountSettings->jid() << "[outgoing] >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
+                                           << Qt::endl
                                            << makeXmlPretty(text);
         break;
     case QXmppLogger::WarningMessage:
-        qCDebug(KAIDAN_CORE_LOG).noquote() << "[client] [warn]" << text;
+        qCDebug(KAIDAN_XMPP_LOG).noquote() << m_accountSettings->jid() << "[client] [warn]" << text;
         break;
     default:
+        qCDebug(KAIDAN_XMPP_LOG).noquote() << m_accountSettings->jid() << "[client] [various]" << text;
         break;
     }
 }
