@@ -11,7 +11,7 @@
 #include <QFileDialog>
 #include <QFutureWatcher>
 #include <QGuiApplication>
-#include <QMimeDatabase>
+#include <QImageReader>
 // KDE
 #include <KFileItem>
 #include <KIO/PreviewJob>
@@ -24,6 +24,7 @@
 #include "FileSharingController.h"
 #include "GroupChatUser.h"
 #include "GroupChatUserDb.h"
+#include "ImageProvider.h"
 #include "MainController.h"
 #include "MediaUtils.h"
 #include "MessageController.h"
@@ -47,12 +48,13 @@ void MessageComposition::setChatController(ChatController *chatController)
         connect(m_chatController, &ChatController::aboutToChangeChat, this, [this]() {
             if (m_chatController->account()) {
                 saveDraft();
+                setIsDraft(false);
                 clear();
             }
         });
 
         connect(m_chatController, &ChatController::chatChanged, this, [this]() {
-            loadDraft().then(this, [this]() {
+            loadDraft().then([this]() {
                 if (const auto messageBodyToForward = m_chatController->messageBodyToForward(); !messageBodyToForward.isEmpty()) {
                     const auto forwardedMessage = [this, messageBodyToForward]() {
                         const QString quotedBody = m_chatController->account()->settings()->displayName() + u":\n" + QmlUtils::quote(messageBodyToForward);
@@ -212,7 +214,7 @@ void MessageComposition::send()
     message.receiptRequested = true;
 
     // generate file IDs if needed
-    if (m_fileSelectionModel->hasFiles()) {
+    if (!message.files.isEmpty()) {
         message.fileGroupId = MessageDb::instance()->newFileGroupId();
 
         for (auto &file : message.files) {
@@ -225,42 +227,7 @@ void MessageComposition::send()
     // add pending message to database
     MessageDb::instance()->addMessage(message, MessageOrigin::UserInput);
 
-    if (m_fileSelectionModel->hasFiles()) {
-        // upload files and send message after uploading
-
-        // whether to symmetrically encrypt the files
-        bool encrypt = message.encryption != Encryption::NoEncryption;
-
-        // upload files
-        m_fileSharingController->sendFiles(message.files, encrypt).then(this, [this, message = std::move(message)](auto result) mutable {
-            if (auto files = std::get_if<QList<File>>(&result)) {
-                // uploading succeeded
-
-                // set updated files with new metadata and uploaded sources
-                message.files = std::move(*files);
-
-                // update message in database
-                MessageDb::instance()->updateMessage(message.accountJid, message.chatJid, message.id, [files = message.files](auto &message) {
-                    message.files = files;
-                });
-
-                // send message with file sources
-                m_messageController->sendPendingMessage(std::move(message));
-            } else {
-                // uploading did not succeed
-                auto errorText = std::get<QXmppError>(std::move(result)).description;
-
-                // set error text in database
-                MessageDb::instance()->updateMessage(message.accountJid, message.chatJid, message.id, [errorText](auto &message) {
-                    message.errorText = tr("Upload failed: %1").arg(errorText);
-                });
-            }
-        });
-        m_fileSelectionModel->clear();
-    } else {
-        // directly send message
-        m_messageController->sendPendingMessage(std::move(message));
-    }
+    m_messageController->sendPendingMessage(std::move(message));
 
     reset();
 }
@@ -295,15 +262,11 @@ void MessageComposition::correct()
 
                 message.deliveryState = Enums::DeliveryState::Pending;
 
-                runOnThread(this, [this, message]() mutable {
+                QMetaObject::invokeMethod(this, [this, message]() mutable {
                     if (m_connection->state() == Enums::ConnectionState::StateConnected) {
-                        // the trick with the time is important for the servers
-                        // this way they can tell which version of the message is the latest
-                        message.timestamp = QDateTime::currentDateTimeUtc();
-
                         auto sendMessage = [this](Message &&message, const QList<QString> &encryptionJids = {}) {
                             m_messageController->send(message.toQXmpp(), message.encryption, encryptionJids)
-                                .then(this, [accountJid = message.accountJid, chatJid = message.chatJid, messageId = message.id](QXmpp::SendResult &&result) {
+                                .then([accountJid = message.accountJid, chatJid = message.chatJid, messageId = message.id](QXmpp::SendResult &&result) {
                                     if (std::holds_alternative<QXmppError>(result)) {
                                         // TODO store in the database only error codes, assign text messages right in the
                                         // QML
@@ -329,8 +292,6 @@ void MessageComposition::correct()
                         }
                     }
                 });
-            } else if (message.replaceId.isEmpty()) {
-                message.timestamp = QDateTime::currentDateTimeUtc();
             }
         });
 
@@ -347,8 +308,9 @@ void MessageComposition::clear()
     setReplyQuote({});
     setOriginalBody({});
     setSpoiler(false);
-    setIsDraft(false);
     setIsForwarding(false);
+
+    m_fileSelectionModel->clear();
 }
 
 void MessageComposition::setReply(Message &message,
@@ -454,7 +416,6 @@ void MessageComposition::saveDraft()
                                                           message.isSpoiler = isSpoiler;
                                                           message.spoilerHint = spoilerHint;
                                                       });
-            setIsDraft(true);
         } else {
             MessageDb::instance()->removeDraftMessage(m_chatController->account()->settings()->jid(), m_chatController->jid());
         }
@@ -471,8 +432,6 @@ void MessageComposition::saveDraft()
         message.deliveryState = DeliveryState::Draft;
 
         MessageDb::instance()->addDraftMessage(message);
-
-        setIsDraft(true);
     }
 }
 
@@ -507,7 +466,7 @@ QHash<int, QByteArray> FileSelectionModel::roleNames() const
         {Description, QByteArrayLiteral("description")},
         {Size, QByteArrayLiteral("size")},
         {LocalFileUrl, QByteArrayLiteral("localFileUrl")},
-        {PreviewImage, QByteArrayLiteral("previewImage")},
+        {PreviewImageUrl, QByteArrayLiteral("previewImageUrl")},
         {Type, QByteArrayLiteral("type")},
     };
     return roles;
@@ -544,12 +503,8 @@ QVariant FileSelectionModel::data(const QModelIndex &index, int role) const
         return tr("Unknown size");
     case LocalFileUrl:
         return file.localFileUrl();
-    case PreviewImage:
-        if (const auto previewImage = file.previewImage(); !previewImage.isNull()) {
-            return previewImage;
-        }
-
-        return file.mimeTypeIcon();
+    case PreviewImageUrl:
+        return ImageProvider::instance()->generatedFileImageUrl(file);
     case Type:
         return QVariant::fromValue(file.type());
     }
@@ -598,27 +553,27 @@ void FileSelectionModel::addFile(const QUrl &localFileUrl, bool isNew)
     file.mimeType = MediaUtils::mimeDatabase().mimeTypeForFile(localPath);
     file.size = fileInfo.size();
     file.isNew = isNew;
+    file.transferOutgoing = true;
 
-    if (const auto image = QImage(localPath); !image.isNull()) {
-        file.width = image.width();
-        file.height = image.height();
+    if (auto image = QImageReader(localPath); image.canRead()) {
+        const auto size = [&image]() {
+            QSize size = image.size();
+
+            if (!size.isValid()) {
+                size = image.read().size();
+            }
+
+            return size;
+        }();
+
+        file.width = size.width();
+        file.height = size.height();
     }
 
-    auto insertFile = [this](File &&file) mutable {
-        beginInsertRows({}, m_files.size(), m_files.size());
-        m_files.append(std::move(file));
-        endInsertRows();
-    };
-
-    if (file.type() == MessageType::MessageVideo) {
-        MediaUtils::generateThumbnail(file.localFileUrl(), file.mimeTypeName(), VIDEO_THUMBNAIL_EDGE_PIXEL_COUNT)
-            .then(this, [file, insertFile](const QByteArray &thumbnail) mutable {
-                file.thumbnail = thumbnail;
-                insertFile(std::move(file));
-            });
-    } else {
-        insertFile(std::move(file));
-    }
+    const int row = m_files.size();
+    beginInsertRows({}, row, row);
+    m_files.append(std::move(file));
+    endInsertRows();
 }
 
 void FileSelectionModel::removeFile(int index)
@@ -632,7 +587,7 @@ void FileSelectionModel::removeFile(int index)
 
 void FileSelectionModel::deleteNewFiles()
 {
-    std::for_each(m_files.cbegin(), m_files.cend(), deleteNewFile);
+    std::ranges::for_each(m_files, deleteNewFile);
 }
 
 void FileSelectionModel::clear()
@@ -665,7 +620,7 @@ const QList<File> &FileSelectionModel::files() const
 void FileSelectionModel::deleteNewFile(const File &file)
 {
     if (file.isNew) {
-        MediaUtils::deleteFile(file.localFileUrl());
+        QFile::remove(file.localFilePath);
     }
 }
 
