@@ -73,22 +73,20 @@ MessageController::MessageController(AccountSettings *accountSettings,
 
     connect(m_fileSharingController, &FileSharingController::filesUploadedForPendingMessage, this, &MessageController::sendPendingMessageWithUploadedFiles);
 
-    runOnThread(m_client, [this]() {
-        connect(m_client, &QXmppClient::messageReceived, this, [this](const QXmppMessage &msg) {
-            handleMessage(msg, MessageOrigin::Stream);
-        });
+    connect(m_client, &QXmppClient::messageReceived, this, [this](const QXmppMessage &msg) {
+        handleMessage(msg, MessageOrigin::Stream);
+    });
 
-        connect(m_messageReceiptManager, &QXmppMessageReceiptManager::messageDelivered, this, [this](const QString &jid, const QString &messageId) {
-            const auto accountJid = m_accountSettings->jid();
-            const auto chatJid = QXmppUtils::jidToBareJid(jid);
+    connect(m_messageReceiptManager, &QXmppMessageReceiptManager::messageDelivered, this, [this](const QString &jid, const QString &messageId) {
+        const auto accountJid = m_accountSettings->jid();
+        const auto chatJid = QXmppUtils::jidToBareJid(jid);
 
-            MessageDb::instance()->updateMessage(accountJid, chatJid, messageId, [accountJid, chatJid](Message &message) {
-                // Only the recipient of the message is allowed to confirm its delivery.
-                if (message.accountJid == accountJid && message.chatJid == chatJid) {
-                    message.deliveryState = Enums::DeliveryState::Delivered;
-                    message.errorText.clear();
-                }
-            });
+        MessageDb::instance()->updateMessage(accountJid, chatJid, messageId, [accountJid, chatJid](Message &message) {
+            // Only the recipient of the message is allowed to confirm its delivery.
+            if (message.accountJid == accountJid && message.chatJid == chatJid) {
+                message.deliveryState = Enums::DeliveryState::Delivered;
+                message.errorText.clear();
+            }
         });
     });
 }
@@ -109,26 +107,20 @@ QFuture<QXmpp::SendResult> MessageController::send(QXmppMessage &&message, Encry
             message.addHint(QXmppMessage::Store);
         }
 
-        runOnThread(m_client, [this, promise, message]() mutable {
-            m_client->send(std::move(message)).then(this, [=](QXmpp::SendResult &&result) mutable {
-                reportFinishedResult(*promise, result);
-            });
+        m_client->send(std::move(message)).then(this, [=](QXmpp::SendResult &&result) mutable {
+            reportFinishedResult(*promise, result);
         });
     } else {
         if (encryptionJids.isEmpty()) {
-            runOnThread(m_client, [this, promise, message]() mutable {
-                m_client->sendSensitive(std::move(message)).then(this, [=](QXmpp::SendResult &&result) mutable {
-                    reportFinishedResult(*promise, result);
-                });
+            m_client->sendSensitive(std::move(message)).then(this, [=](QXmpp::SendResult &&result) mutable {
+                reportFinishedResult(*promise, result);
             });
         } else {
             QXmppSendStanzaParams params;
             params.setEncryptionJids(encryptionJids);
 
-            runOnThread(m_client, [this, promise, message, params]() mutable {
-                m_client->sendSensitive(std::move(message), params).then(this, [=](QXmpp::SendResult &&result) mutable {
-                    reportFinishedResult(*promise, result);
-                });
+            m_client->sendSensitive(std::move(message), params).then(this, [=](QXmpp::SendResult &&result) mutable {
+                reportFinishedResult(*promise, result);
             });
         }
     }
@@ -163,11 +155,11 @@ void MessageController::sendMessageWithUndecidedEncryption(Message message)
         if (rosterItem->isGroupChat()) {
             GroupChatUserDb::instance()->userJids(accountJid, chatJid).then(this, [this, processMessage](QList<QString> &&encryptionJids) mutable {
                 if (!encryptionJids.isEmpty()) {
-                    m_encryptionController->hasUsableDevices(encryptionJids).then(this, processMessage);
+                    m_encryptionController->hasUsableDevices(encryptionJids).then(processMessage);
                 }
             });
         } else {
-            m_encryptionController->hasUsableDevices({chatJid}).then(this, processMessage);
+            m_encryptionController->hasUsableDevices({chatJid}).then(processMessage);
         }
     }
 }
@@ -274,42 +266,37 @@ void MessageController::sendPendingMessageWithUploadedFiles(Message message)
 {
     auto sendMessage = [this](Message &&message, const QList<QString> &encryptionJids = {}) {
         send(message.toQXmpp(), message.encryption, encryptionJids)
-            .then(this,
-                  [this,
+            .then([this,
                    accountJid = m_accountSettings->jid(),
                    chatJid = message.chatJid,
                    messageId = message.id,
                    fileFallbackMessages = message.fileFallbackMessages(),
                    encryption = message.encryption,
                    encryptionJids](QXmpp::SendResult result) mutable {
-                      if (const auto error = std::get_if<QXmppError>(&result)) {
-                          qCWarning(KAIDAN_CORE_LOG) << "Could not send message:" << error->description;
-                          Q_EMIT MainController::instance()->passiveNotificationRequested(tr("Message could not be sent"));
+                if (const auto error = std::get_if<QXmppError>(&result)) {
+                    qCWarning(KAIDAN_CORE_LOG) << "Could not send message:" << error->description;
+                    Q_EMIT MainController::instance()->passiveNotificationRequested(tr("Message could not be sent"));
 
-                          MessageDb::instance()->updateMessage(accountJid, chatJid, messageId, [](Message &msg) {
-                              msg.deliveryState = Enums::DeliveryState::Error;
-                              msg.errorText = tr("Message could not be sent.");
-                          });
-                      } else {
-                          MessageDb::instance()->updateMessage(accountJid, chatJid, messageId, [](Message &msg) {
-                              msg.deliveryState = Enums::DeliveryState::Sent;
-                              msg.errorText.clear();
-                          });
+                    MessageDb::instance()->updateMessage(accountJid, chatJid, messageId, [](Message &msg) {
+                        msg.deliveryState = Enums::DeliveryState::Error;
+                        msg.errorText = tr("Message could not be sent.");
+                    });
+                } else {
+                    MessageDb::instance()->updateMessage(accountJid, chatJid, messageId, [](Message &msg) {
+                        msg.deliveryState = Enums::DeliveryState::Sent;
+                        msg.errorText.clear();
+                    });
 
-                          for (auto &fileFallbackMessage : fileFallbackMessages) {
-                              // TODO: Track sending of fallback messages individually
-                              // Needed for the case when the success differs between the main message
-                              // and the fallback messages.
-                              send(std::move(fileFallbackMessage), encryption, encryptionJids);
-                          }
-                      }
-                  });
+                    for (auto &fileFallbackMessage : fileFallbackMessages) {
+                        // TODO: Track sending of fallback messages individually
+                        // Needed for the case when the success differs between the main message
+                        // and the fallback messages.
+                        send(std::move(fileFallbackMessage), encryption, encryptionJids);
+                    }
+                }
+            });
     };
 
-    if (!message.replaceId.isEmpty()) {
-        message.timestamp = QDateTime::currentDateTimeUtc();
-    }
-
     const auto isGroupChatMessage = message.isGroupChatMessage();
 
     message.receiptRequested = !isGroupChatMessage;
@@ -333,13 +320,8 @@ QFuture<bool> MessageController::retrieveBacklogMessages(const QString &jid, boo
     queryLimit.setBefore(oldestMessageStanzaId);
 
     // TODO: Find out why sometimes messages are not correctly retrieved (maybe not even correctly parsed, at least with Tigase)
-    callRemoteTask(
-        m_mamManager,
-        [this, jid, isGroupChat, queryLimit]() {
-            return std::pair{m_mamManager->retrieveMessages(isGroupChat ? jid : QString{}, {}, isGroupChat ? QString{} : jid, {}, {}, queryLimit), this};
-        },
-        this,
-        [this, promise, jid](QXmppMamManager::RetrieveResult &&result) mutable {
+    m_mamManager->retrieveMessages(isGroupChat ? jid : QString{}, {}, isGroupChat ? QString{} : jid, {}, {}, queryLimit)
+        .then(this, [this, promise, jid](QXmppMamManager::RetrieveResult &&result) mutable {
             if (auto error = std::get_if<QXmppError>(&result)) {
                 qCDebug(KAIDAN_CORE_LOG) << "Could not retrieve backlog messages:" << error->description;
                 reportFinishedResult(*promise, false);
@@ -397,15 +379,13 @@ void MessageController::handleRosterReceived(const QString &accountJid)
             }
         });
 
-        if (const auto rosterItems = RosterModel::instance()->items(m_accountSettings->jid()); !rosterItems.isEmpty()) {
-            // TODO: Check whether the own server supports archiving group chat messages and skip the retrieval for each group chat in that case
-            for (const auto &rosterItem : std::as_const(rosterItems)) {
-                if (rosterItem.isGroupChat() && !rosterItem.isDeletedGroupChat()) {
-                    if (rosterItem.latestGroupChatMessageStanzaId.isEmpty()) {
-                        retrieveAllMessages(rosterItem.jid);
-                    } else {
-                        retrieveCatchUpMessages(rosterItem.latestGroupChatMessageStanzaId, rosterItem.jid);
-                    }
+        // TODO: Check whether the own server supports archiving group chat messages and skip the retrieval for each group chat in that case
+        for (const auto &rosterItem : RosterModel::instance()->items(m_accountSettings->jid())) {
+            if (rosterItem.isGroupChat() && !rosterItem.isDeletedGroupChat()) {
+                if (rosterItem.latestGroupChatMessageStanzaId.isEmpty()) {
+                    retrieveAllMessages(rosterItem.jid);
+                } else {
+                    retrieveCatchUpMessages(rosterItem.latestGroupChatMessageStanzaId, rosterItem.jid);
                 }
             }
         }
@@ -416,10 +396,8 @@ void MessageController::handleRosterReceived(const QString &accountJid)
 
 void MessageController::retrieveInitialMessages()
 {
-    if (const auto rosterItems = RosterModel::instance()->items(m_accountSettings->jid()); !rosterItems.isEmpty()) {
-        for (const auto &rosterItem : std::as_const(rosterItems)) {
-            retrieveInitialMessage(rosterItem.jid, rosterItem.isGroupChat());
-        }
+    for (const auto &rosterItem : RosterModel::instance()->items(m_accountSettings->jid())) {
+        retrieveInitialMessage(rosterItem.jid, rosterItem.isGroupChat());
     }
 }
 
@@ -429,13 +407,8 @@ void MessageController::retrieveInitialMessage(const QString &jid, bool isGroupC
     queryLimit.setMax(1);
     queryLimit.setBefore(offsetMessageId);
 
-    callRemoteTask(
-        m_mamManager,
-        [this, jid, isGroupChat, queryLimit]() {
-            return std::pair{m_mamManager->retrieveMessages(isGroupChat ? jid : QString{}, {}, isGroupChat ? QString{} : jid, {}, {}, queryLimit), this};
-        },
-        this,
-        [this, jid, isGroupChat](QXmppMamManager::RetrieveResult &&result) mutable {
+    m_mamManager->retrieveMessages(isGroupChat ? jid : QString{}, {}, isGroupChat ? QString{} : jid, {}, {}, queryLimit)
+        .then(this, [this, jid, isGroupChat](QXmppMamManager::RetrieveResult &&result) mutable {
             if (auto error = std::get_if<QXmppError>(&result)) {
                 qCDebug(KAIDAN_CORE_LOG) << "Could not retrieve initial message:" << error->description;
             } else {
@@ -456,26 +429,20 @@ void MessageController::retrieveInitialMessage(const QString &jid, bool isGroupC
 
 void MessageController::retrieveAllMessages(const QString &groupChatJid)
 {
-    callRemoteTask(
-        m_mamManager,
-        [this, groupChatJid]() {
-            return std::pair{m_mamManager->retrieveMessages(groupChatJid), this};
-        },
-        this,
-        [this, groupChatJid](QXmppMamManager::RetrieveResult &&result) mutable {
-            if (auto error = std::get_if<QXmppError>(&result)) {
-                qCDebug(KAIDAN_CORE_LOG) << "Could not retrieve all messages:" << error->description;
-            } else {
-                const auto retrievedMessages = std::get<QXmppMamManager::RetrievedMessages>(std::move(result));
+    m_mamManager->retrieveMessages(groupChatJid).then(this, [this, groupChatJid](QXmppMamManager::RetrieveResult &&result) mutable {
+        if (auto error = std::get_if<QXmppError>(&result)) {
+            qCDebug(KAIDAN_CORE_LOG) << "Could not retrieve all messages:" << error->description;
+        } else {
+            const auto retrievedMessages = std::get<QXmppMamManager::RetrievedMessages>(std::move(result));
 
-                for (const auto &message : std::as_const(retrievedMessages.messages)) {
-                    handleMessage(message, MessageOrigin::MamCatchUp);
+            for (const auto &message : std::as_const(retrievedMessages.messages)) {
+                handleMessage(message, MessageOrigin::MamCatchUp);
 
-                    // Send delivery receipts for the retrieved message.
-                    m_messageReceiptManager->handleMessage(message);
-                }
+                // Send delivery receipts for the retrieved message.
+                m_messageReceiptManager->handleMessage(message);
             }
-        });
+        }
+    });
 }
 
 void MessageController::retrieveCatchUpMessages(const QString &latestMessageStanzaId, const QString &groupChatJid)
@@ -483,26 +450,20 @@ void MessageController::retrieveCatchUpMessages(const QString &latestMessageStan
     QXmppResultSetQuery queryLimit;
     queryLimit.setAfter(latestMessageStanzaId);
 
-    callRemoteTask(
-        m_mamManager,
-        [this, groupChatJid, queryLimit]() {
-            return std::pair{m_mamManager->retrieveMessages(groupChatJid, {}, {}, {}, {}, queryLimit), this};
-        },
-        this,
-        [this, groupChatJid](QXmppMamManager::RetrieveResult &&result) mutable {
-            if (auto error = std::get_if<QXmppError>(&result)) {
-                qCDebug(KAIDAN_CORE_LOG) << "Could not retrieve catch-up messages:" << error->description;
-            } else {
-                const auto retrievedMessages = std::get<QXmppMamManager::RetrievedMessages>(std::move(result));
+    m_mamManager->retrieveMessages(groupChatJid, {}, {}, {}, {}, queryLimit).then(this, [this, groupChatJid](QXmppMamManager::RetrieveResult &&result) mutable {
+        if (auto error = std::get_if<QXmppError>(&result)) {
+            qCDebug(KAIDAN_CORE_LOG) << "Could not retrieve catch-up messages:" << error->description;
+        } else {
+            const auto retrievedMessages = std::get<QXmppMamManager::RetrievedMessages>(std::move(result));
 
-                for (const auto &message : std::as_const(retrievedMessages.messages)) {
-                    handleMessage(message, MessageOrigin::MamCatchUp);
+            for (const auto &message : std::as_const(retrievedMessages.messages)) {
+                handleMessage(message, MessageOrigin::MamCatchUp);
 
-                    // Send delivery receipts for the retrieved message.
-                    m_messageReceiptManager->handleMessage(message);
-                }
+                // Send delivery receipts for the retrieved message.
+                m_messageReceiptManager->handleMessage(message);
             }
-        });
+        }
+    });
 }
 
 void MessageController::handleMessage(const QXmppMessage &msg, MessageOrigin origin)
@@ -563,7 +524,7 @@ void MessageController::handleMessage(const QXmppMessage &msg, MessageOrigin ori
     // That is needed if the own server archives group chat messages resulting in a second stanza
     // ID added by the own server to the group chat message.
     if (receivedFromGroupChat) {
-        const auto itr = std::find_if(stanzaIds.cbegin(), stanzaIds.cend(), [senderJid](const QXmppStanzaId &stanzaId) {
+        const auto itr = std::ranges::find_if(stanzaIds, [senderJid](const QXmppStanzaId &stanzaId) {
             return stanzaId.by == senderJid;
         });
 
@@ -797,10 +758,9 @@ bool MessageController::handleReaction(const QXmppMessage &message, const QStrin
 
                                                      // Add new reactions.
                                                      for (const auto &receivedEmoji : std::as_const(receivedEmojis)) {
-                                                         const auto reactionNew =
-                                                             std::none_of(reactions.cbegin(), reactions.cend(), [&](const MessageReaction &reaction) {
-                                                                 return reaction.emoji == receivedEmoji;
-                                                             });
+                                                         const auto reactionNew = std::ranges::none_of(reactions, [&](const MessageReaction &reaction) {
+                                                             return reaction.emoji == receivedEmoji;
+                                                         });
 
                                                          if (reactionNew) {
                                                              MessageReaction reaction;
@@ -960,7 +920,7 @@ void MessageController::parseSharedFiles(const QXmppMessage &message, Message &m
                 const auto &bobData = message.bitsOfBinaryData();
                 if (!fileShare.metadata().thumbnails().empty()) {
                     auto cid = QXmppBitsOfBinaryContentId::fromCidUrl(fileShare.metadata().thumbnails().front().uri());
-                    const auto thumbnailData = std::find_if(bobData.begin(), bobData.end(), [&](auto bobBlob) {
+                    const auto thumbnailData = std::ranges::find_if(bobData, [&](auto bobBlob) {
                         return bobBlob.cid() == cid;
                     });
 
@@ -1054,7 +1014,7 @@ void MessageController::sendPendingMessageReactions()
                                                                        Encryption::Enum encryption = Encryption::NoEncryption,
                                                                        const QList<QString> &encryptionJids = {}) {
                     sendMessageReaction(chatJid, messageId, isGroupChat, emojis, encryption, encryptionJids)
-                        .then(this, [this, chatJid, messageId, accountJid = m_accountSettings->jid()](QXmpp::SendResult &&result) {
+                        .then([this, chatJid, messageId, accountJid = m_accountSettings->jid()](QXmpp::SendResult &&result) {
                             if (const auto error = std::get_if<QXmppError>(&result)) {
                                 Q_EMIT MainController::instance()->passiveNotificationRequested(tr("Reaction could not be sent: %1").arg(error->description));
 
@@ -1093,7 +1053,7 @@ void MessageController::sendPendingMessageReactions()
                             .then(this, [this, sendReaction, encryption](QList<QString> &&encryptionJids) mutable {
                                 if (!encryptionJids.isEmpty()) {
                                     m_encryptionController->hasUsableDevices(encryptionJids)
-                                        .then(this, [sendReaction, encryption, encryptionJids](bool hasUsableDevices) mutable {
+                                        .then([sendReaction, encryption, encryptionJids](bool hasUsableDevices) mutable {
                                             if (hasUsableDevices) {
                                                 sendReaction(true, encryption, encryptionJids);
                                             } else {
@@ -1103,7 +1063,7 @@ void MessageController::sendPendingMessageReactions()
                                 }
                             });
                     } else {
-                        m_encryptionController->hasUsableDevices({chatJid}).then(this, [sendReaction, encryption](bool hasUsableDevices) mutable {
+                        m_encryptionController->hasUsableDevices({chatJid}).then([sendReaction, encryption](bool hasUsableDevices) mutable {
                             if (hasUsableDevices) {
                                 sendReaction(false, encryption);
                             } else {
@@ -1135,7 +1095,7 @@ void MessageController::sendPendingReadMarkers()
                             .then(this, [this, chatJid, messageId, encryption](QList<QString> &&encryptionJids) mutable {
                                 if (!encryptionJids.isEmpty()) {
                                     m_encryptionController->hasUsableDevices(encryptionJids)
-                                        .then(this, [this, chatJid, messageId, encryption, encryptionJids](bool hasUsableDevices) mutable {
+                                        .then([this, chatJid, messageId, encryption, encryptionJids](bool hasUsableDevices) mutable {
                                             if (hasUsableDevices) {
                                                 sendReadMarker(chatJid, messageId, encryption, encryptionJids);
                                             } else {
@@ -1145,7 +1105,7 @@ void MessageController::sendPendingReadMarkers()
                                 }
                             });
                     } else {
-                        m_encryptionController->hasUsableDevices({chatJid}).then(this, [this, chatJid, messageId, encryption](bool hasUsableDevices) mutable {
+                        m_encryptionController->hasUsableDevices({chatJid}).then([this, chatJid, messageId, encryption](bool hasUsableDevices) mutable {
                             if (hasUsableDevices) {
                                 sendReadMarker(chatJid, messageId, encryption);
                             } else {
