@@ -9,8 +9,6 @@
 // Qt
 #include <QObject>
 #include <QUrl>
-// QXmpp
-#include <QXmppMessage.h>
 
 class QGeoCoordinate;
 
@@ -176,11 +174,6 @@ public:
      */
     Q_INVOKABLE static QColor userColor(const QString &id, const QString &name);
 
-    /**
-     * Returns a human-readable string describing the state of the chat.
-     */
-    Q_INVOKABLE static QString chatStateDescription(const QString &displayName, const QXmppMessage::State state);
-
     Q_INVOKABLE static QString osmUserAgent();
 
     Q_INVOKABLE static QString geoUri(const QGeoCoordinate &geoCoordinate);
