@@ -14,7 +14,7 @@
 #include <QXmppStanza.h>
 
 class AccountSettings;
-class ClientWorker;
+class ClientController;
 class Connection;
 class DataFormModel;
 class EncryptionController;
@@ -57,10 +57,10 @@ public:
                            Connection *connection,
                            EncryptionController *encryptionController,
                            VCardController *vCardController,
-                           ClientWorker *clientWorker,
+                           ClientController *clientController,
                            QObject *parent = nullptr);
 
-    QFuture<bool> registerOnConnectEnabled();
+    bool registerOnConnectEnabled() const;
 
     /**
      * Connects to the server and requests a data form for account registration.
@@ -237,7 +237,7 @@ private:
     Connection *const m_connection;
     EncryptionController *const m_encryptionController;
     VCardController *const m_vCardController;
-    ClientWorker *const m_clientWorker;
+    ClientController *const m_clientController;
     QXmppClient *const m_client;
     QXmppRegistrationManager *const m_manager;
 
