@@ -163,15 +163,15 @@ QStringList RosterModel::groups() const
 {
     QStringList groups;
 
-    std::for_each(m_items.cbegin(), m_items.cend(), [&groups](const RosterItem &item) {
-        std::for_each(item.groups.cbegin(), item.groups.cend(), [&](const QString &group) {
+    std::ranges::for_each(m_items, [&groups](const RosterItem &item) {
+        std::ranges::for_each(item.groups, [&](const QString &group) {
             if (!groups.contains(group)) {
                 groups.append(group);
             }
         });
     });
 
-    std::sort(groups.begin(), groups.end());
+    std::ranges::sort(groups);
 
     return groups;
 }
@@ -333,7 +333,7 @@ void RosterModel::handleItemsFetched(const QList<RosterItem> &items)
 {
     beginResetModel();
     m_items = items;
-    std::sort(m_items.begin(), m_items.end());
+    std::ranges::sort(m_items);
     endResetModel();
 
     for (const auto &item : std::as_const(m_items)) {
@@ -457,7 +457,7 @@ void RosterModel::handleAccountNotificationRuleChanged()
 
 void RosterModel::handleMessageAdded(const Message &message, MessageOrigin origin)
 {
-    auto itr = std::find_if(m_items.begin(), m_items.end(), [&message](const RosterItem &item) {
+    auto itr = std::ranges::find_if(m_items, [&message](const RosterItem &item) {
         return item.accountJid == message.accountJid && item.jid == message.chatJid;
     });
 
@@ -466,34 +466,36 @@ void RosterModel::handleMessageAdded(const Message &message, MessageOrigin origi
         return;
     }
 
-    updateLastMessage(itr, message).then(this, [this, origin, itr](QList<int> &&changedRoles) mutable {
-        switch (origin) {
-        case MessageOrigin::Stream:
-        case MessageOrigin::UserInput:
-        case MessageOrigin::MamCatchUp:
-            MessageDb::instance()
-                ->latestContactMessageCount(itr->accountJid, itr->jid, itr->lastReadContactMessageId)
-                .then([this, itr, changedRoles](int unreadMessageCount) mutable {
-                    if (unreadMessageCount != itr->unreadMessageCount) {
-                        itr->unreadMessageCount = unreadMessageCount;
-                        changedRoles << int(UnreadMessageCountRole);
-                    }
+    switch (origin) {
+    case MessageOrigin::Stream:
+    case MessageOrigin::UserInput:
+    case MessageOrigin::MamCatchUp:
+        MessageDb::instance()
+            ->latestContactMessageCount(itr->accountJid, itr->jid, itr->lastReadContactMessageId)
+            .then([this, itr, changedRoles = updateLastMessage(itr, message)](int unreadMessageCount) mutable {
+                if (unreadMessageCount != itr->unreadMessageCount) {
+                    itr->unreadMessageCount = unreadMessageCount;
+                    changedRoles << int(UnreadMessageCountRole);
+                }
 
-                    if (!changedRoles.isEmpty()) {
-                        updateOnMessageChange(itr, changedRoles);
-                    }
-                });
-            break;
-        case MessageOrigin::MamBacklog:
-        case MessageOrigin::MamInitial:
-            break;
+                if (!changedRoles.isEmpty()) {
+                    updateOnMessageChange(itr, changedRoles);
+                }
+            });
+        break;
+    case MessageOrigin::MamBacklog:
+        break;
+    case MessageOrigin::MamInitial:
+        if (const auto changedRoles = updateLastMessage(itr, message); !changedRoles.isEmpty()) {
+            updateOnMessageChange(itr, changedRoles);
         }
-    });
+        break;
+    }
 }
 
 void RosterModel::handleMessageUpdated(const Message &message)
 {
-    auto itr = std::find_if(m_items.begin(), m_items.end(), [&message](const RosterItem &item) {
+    auto itr = std::ranges::find_if(m_items, [&message](const RosterItem &item) {
         return item.accountJid == message.accountJid && item.jid == message.chatJid;
     });
 
@@ -502,8 +504,9 @@ void RosterModel::handleMessageUpdated(const Message &message)
         return;
     }
 
-    updateLastMessage(itr, message).then(this, [this, itr](QList<int> &&changedRoles) mutable {
-        MessageDb::instance()->markedMessageCount(itr->accountJid, itr->jid).then([this, itr, changedRoles](int markedMessageCount) mutable {
+    MessageDb::instance()
+        ->markedMessageCount(itr->accountJid, itr->jid)
+        .then([this, itr, changedRoles = updateLastMessage(itr, message)](int markedMessageCount) mutable {
             if (markedMessageCount != itr->markedMessageCount) {
                 itr->markedMessageCount = markedMessageCount;
                 changedRoles << int(MarkedMessageCountRole);
@@ -513,12 +516,11 @@ void RosterModel::handleMessageUpdated(const Message &message)
                 updateOnMessageChange(itr, changedRoles);
             }
         });
-    });
 }
 
 void RosterModel::handleMessageRemoved(const Message &newLastMessage)
 {
-    auto itr = std::find_if(m_items.begin(), m_items.end(), [&newLastMessage](const RosterItem &item) {
+    auto itr = std::ranges::find_if(m_items, [&newLastMessage](const RosterItem &item) {
         return item.accountJid == newLastMessage.accountJid && item.jid == newLastMessage.chatJid;
     });
 
@@ -527,32 +529,30 @@ void RosterModel::handleMessageRemoved(const Message &newLastMessage)
         return;
     }
 
-    updateLastMessage(itr, newLastMessage, false).then(this, [this, itr](QList<int> &&changedRoles) mutable {
-        MessageDb::instance()
-            ->latestContactMessageCount(itr->accountJid, itr->jid, itr->lastReadContactMessageId)
-            .then([this, itr, changedRoles](int unreadMessageCount) mutable {
-                if (unreadMessageCount != itr->unreadMessageCount) {
-                    itr->unreadMessageCount = unreadMessageCount;
-                    changedRoles << int(UnreadMessageCountRole);
+    MessageDb::instance()
+        ->latestContactMessageCount(itr->accountJid, itr->jid, itr->lastReadContactMessageId)
+        .then([this, itr, changedRoles = updateLastMessage(itr, newLastMessage, false)](int unreadMessageCount) mutable {
+            if (unreadMessageCount != itr->unreadMessageCount) {
+                itr->unreadMessageCount = unreadMessageCount;
+                changedRoles << int(UnreadMessageCountRole);
+            }
+
+            MessageDb::instance()->markedMessageCount(itr->accountJid, itr->jid).then([this, itr, changedRoles](int markedMessageCount) mutable {
+                if (markedMessageCount != itr->markedMessageCount) {
+                    itr->markedMessageCount = markedMessageCount;
+                    changedRoles << int(MarkedMessageCountRole);
                 }
 
-                MessageDb::instance()->markedMessageCount(itr->accountJid, itr->jid).then([this, itr, changedRoles](int markedMessageCount) mutable {
-                    if (markedMessageCount != itr->markedMessageCount) {
-                        itr->markedMessageCount = markedMessageCount;
-                        changedRoles << int(MarkedMessageCountRole);
-                    }
-
-                    if (!changedRoles.isEmpty()) {
-                        updateOnMessageChange(itr, changedRoles);
-                    }
-                });
+                if (!changedRoles.isEmpty()) {
+                    updateOnMessageChange(itr, changedRoles);
+                }
             });
-    });
+        });
 }
 
 void RosterModel::handleDraftMessageAdded(const Message &message)
 {
-    auto itr = std::find_if(m_items.begin(), m_items.end(), [&message](const RosterItem &item) {
+    auto itr = std::ranges::find_if(m_items, [&message](const RosterItem &item) {
         return item.accountJid == message.accountJid && item.jid == message.chatJid;
     });
 
@@ -570,7 +570,7 @@ void RosterModel::handleDraftMessageAdded(const Message &message)
 
 void RosterModel::handleDraftMessageUpdated(const Message &message)
 {
-    auto itr = std::find_if(m_items.begin(), m_items.end(), [&message](const RosterItem &item) {
+    auto itr = std::ranges::find_if(m_items, [&message](const RosterItem &item) {
         return item.accountJid == message.accountJid && item.jid == message.chatJid;
     });
 
@@ -587,7 +587,7 @@ void RosterModel::handleDraftMessageUpdated(const Message &message)
 
 void RosterModel::handleDraftMessageRemoved(const Message &newLastMessage)
 {
-    auto itr = std::find_if(m_items.begin(), m_items.end(), [&newLastMessage](const RosterItem &item) {
+    auto itr = std::ranges::find_if(m_items, [&newLastMessage](const RosterItem &item) {
         return item.accountJid == newLastMessage.accountJid && item.jid == newLastMessage.chatJid;
     });
 
@@ -599,21 +599,26 @@ void RosterModel::handleDraftMessageRemoved(const Message &newLastMessage)
     itr->lastMessageDateTime = newLastMessage.timestamp;
     itr->lastMessageDeliveryState = newLastMessage.deliveryState;
     itr->lastMessage = newLastMessage.previewText();
+    itr->lastMessageIsOwn = newLastMessage.isOwn;
 
-    updateOnDraftMessageChange(itr);
+    QList<int> changedRoles = {int(LastMessageDateTimeRole), int(LastMessageRole), int(LastMessageIsDraftRole), int(LastMessageIsOwnRole)};
+
+    if (itr->isGroupChat()) {
+        itr->lastMessageGroupChatSenderName = newLastMessage.groupChatSenderName;
+        changedRoles.append(int(LastMessageGroupChatSenderNameRole));
+    }
+
+    updateOnMessageChange(itr, changedRoles);
 }
 
-QFuture<QList<int>> RosterModel::updateLastMessage(QList<RosterItem>::Iterator &itr, const Message &message, bool onlyUpdateIfNewerOrAtSameAge)
+QList<int> RosterModel::updateLastMessage(QList<RosterItem>::Iterator &itr, const Message &message, bool onlyUpdateIfNewerOrAtSameAge)
 {
-    auto promise = std::make_shared<QPromise<QList<int>>>();
-    promise->start();
-
     // If desired, only set the new message as the current last message if it is newer than
     // the current one or at the same age.
     // That makes it possible to use the previous message as the new last message if the current
     // last message is empty.
     if (!itr->lastMessage.isEmpty() && (onlyUpdateIfNewerOrAtSameAge && itr->lastMessageDateTime > message.timestamp)) {
-        reportFinishedResult(*promise, {});
+        return {};
     }
 
     // The new message is only set as the current last message if they are different and there
@@ -637,12 +642,10 @@ QFuture<QList<int>> RosterModel::updateLastMessage(QList<RosterItem>::Iterator &
             int(LastMessageDateTimeRole),
         };
 
-        reportFinishedResult(*promise, std::move(changedRoles));
-    } else {
-        reportFinishedResult(*promise, {});
+        return changedRoles;
     }
 
-    return promise->future();
+    return {};
 }
 
 void RosterModel::updateOnMessageChange(QList<RosterItem>::Iterator &itr, const QList<int> &changedRoles)
