@@ -50,6 +50,8 @@
 #endif
 // KDAB
 #include <kdsingleapplication.h>
+// GStreamer
+#include <gst/gst.h>
 // QXmpp
 #include <QXmppClient.h>
 #include <QXmppMixInfoItem.h>
@@ -69,8 +71,10 @@
 #include "AuthenticatedEncryptionKeyModel.h"
 #include "AvatarCache.h"
 #include "Blocking.h"
+#include "CallController.h"
 #include "ChatController.h"
 #include "ChatHintModel.h"
+#include "ChatStateController.h"
 #include "ContactTrustMessageUriGenerator.h"
 #include "CredentialsGenerator.h"
 #include "CredentialsValidator.h"
@@ -97,6 +101,7 @@
 #include "HostCompletionProxyModel.h"
 #include "ImageProvider.h"
 #include "KaidanLog.h"
+#include "Keychain.h"
 #include "MainController.h"
 #include "MediaUtils.h"
 #include "Message.h"
@@ -166,6 +171,7 @@ enum CommandLineParseResult {
     CommandLineError,
     CommandLineVersionRequested,
     CommandLineHelpRequested,
+    CommandLineUnencryptedKeychainRequested,
 };
 
 CommandLineParseResult parseCommandLine(QCommandLineParser &parser, QString *errorMessage)
@@ -176,9 +182,12 @@ CommandLineParseResult parseCommandLine(QCommandLineParser &parser, QString *err
     // add all possible arguments
     QCommandLineOption helpOption = parser.addHelpOption();
     QCommandLineOption versionOption = parser.addVersionOption();
+    QCommandLineOption unencryptedKeychainOption = {{QStringLiteral("u"), QStringLiteral("unencrypted-keychain")},
+                                                    QStringLiteral("Store passwords in an unencrypted file.")};
 #ifndef NDEBUG
     parser.addOption({{QStringLiteral("m"), QStringLiteral("multiple")}, QStringLiteral("Allow multiple instances to be started.")});
 #endif
+    parser.addOption(unencryptedKeychainOption);
     parser.addPositionalArgument(QStringLiteral("xmpp-uri"), QStringLiteral("An XMPP-URI to open (i.e. join a chat)."), QStringLiteral("[xmpp-uri]"));
 
     // parse arguments
@@ -193,6 +202,10 @@ CommandLineParseResult parseCommandLine(QCommandLineParser &parser, QString *err
 
     if (parser.isSet(helpOption))
         return CommandLineHelpRequested;
+
+    if (parser.isSet(unencryptedKeychainOption))
+        return CommandLineUnencryptedKeychainRequested;
+
     // if nothing special happened, return OK
     return CommandLineOk;
 }
@@ -215,13 +228,13 @@ void filterLog(QtMsgType type, const QMessageLogContext &context, const QString
         }
     }
 
-    if (const auto match = std::find_if(filterStrings.cbegin(),
-                                        filterStrings.cend(),
-                                        [&msg](const QString &filter) {
-                                            const auto rx =
-                                                QRegularExpression::fromWildcard(filter, Qt::CaseInsensitive, QRegularExpression::UnanchoredWildcardConversion);
-                                            return msg.contains(rx);
-                                        });
+    if (const auto match =
+            std::ranges::find_if(filterStrings,
+                                 [&msg](const QString &filter) {
+                                     const auto rx =
+                                         QRegularExpression::fromWildcard(filter, Qt::CaseInsensitive, QRegularExpression::UnanchoredWildcardConversion);
+                                     return msg.contains(rx);
+                                 });
         match != filterStrings.cend()) {
         return;
     }
@@ -338,6 +351,7 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     qRegisterMetaType<QList<GroupChatUser>>();
     qRegisterMetaType<std::function<void(GroupChatUser &)>>();
     qRegisterMetaType<GroupChatUserModel *>();
+    qRegisterMetaType<CallController *>();
     // The alias is needed because the type shares its QMetaType::id() with quint32.
     qRegisterMetaType<uint32_t>("uint32_t");
 
@@ -349,13 +363,12 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     qRegisterMetaType<MessageType>();
     qRegisterMetaType<Enums::ConnectionState>();
     qRegisterMetaType<PublicGroupChatModel::CustomRole>();
-    qRegisterMetaType<ClientWorker::ConnectionError>();
+    qRegisterMetaType<ClientController::ConnectionError>();
     qRegisterMetaType<Enums::MessageType>();
     qRegisterMetaType<Presence::Availability>();
     qRegisterMetaType<Enums::DeliveryState>();
     qRegisterMetaType<MessageOrigin>();
     qRegisterMetaType<ProviderModel::Role>();
-    qRegisterMetaType<ChatState::State>();
     qRegisterMetaType<Encryption>();
     qRegisterMetaType<MessageReactionDeliveryState>();
     qRegisterMetaType<FileModel::Role>();
@@ -379,6 +392,13 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     qRegisterMetaType<QXmppMixParticipantItem>();
     qRegisterMetaType<QMultiHash<QString, QByteArray>>();
 
+    // Keychain
+
+    if (!QKeychain::isAvailable()) {
+        qCWarning(KAIDAN_LOG, "Account passwords will be stored in unencrypted file since no password manager is available");
+        QKeychainFuture::setUnencryptedFallback(true);
+    }
+
     //
     // Command line arguments
     //
@@ -397,6 +417,10 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     case CommandLineHelpRequested:
         parser.showHelp();
         return 0;
+    case CommandLineUnencryptedKeychainRequested:
+        qCWarning(KAIDAN_LOG, "Passwords will be stored in an unencrypted file");
+        QKeychainFuture::setUnencryptedFallback(true);
+        Q_FALLTHROUGH();
     case CommandLineOk:
         break;
     }
@@ -486,6 +510,10 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     }
 #endif
 
+    // Allow importing org.freedesktop.gstreamer.Qt6GLVideoItem and using GstGLQt6VideoItem in QML.
+    gst_init(&argc, &argv);
+    gst_element_factory_make("qml6glsink", NULL);
+
     // QML type bindings
     qmlRegisterType<StatusBar>(APPLICATION_ID, 1, 0, "StatusBar");
     qmlRegisterType<EmojiModel>(APPLICATION_ID, 1, 0, "EmojiModel");
@@ -531,26 +559,21 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     qmlRegisterType<ProviderFilterModel>(APPLICATION_ID, 1, 0, "ProviderFilterModel");
 
     // Q_OBJECT
-    qmlRegisterUncreatableType<Account>(APPLICATION_ID, 1, 0, "Account", QStringLiteral("Cannot create object; only enums defined!"));
-    qmlRegisterUncreatableType<AccountSettings>(APPLICATION_ID, 1, 0, "AccountSettings", QStringLiteral("Cannot create object"));
+    qmlRegisterUncreatableType<Account>(APPLICATION_ID, 1, 0, "Account", QStringLiteral("Not creatable from QML"));
+    qmlRegisterUncreatableType<AccountSettings>(APPLICATION_ID, 1, 0, "AccountSettings", QStringLiteral("Not creatable from QML"));
     qmlRegisterUncreatableType<AtmController>(APPLICATION_ID, 1, 0, "AtmController", QStringLiteral("Cannot create object; only enums defined!"));
     qmlRegisterUncreatableType<QAbstractItemModel>(APPLICATION_ID, 1, 0, "QAbstractItemModel", QStringLiteral("Used by proxy models"));
-    qmlRegisterUncreatableType<ClientWorker>(APPLICATION_ID, 1, 0, "ClientWorker", QStringLiteral("Cannot create object; only enums defined!"));
+    qmlRegisterUncreatableType<ClientController>(APPLICATION_ID, 1, 0, "ClientController", QStringLiteral("Cannot create object; only enums defined!"));
     qmlRegisterUncreatableType<DataFormModel>(APPLICATION_ID, 1, 0, "DataFormModel", QStringLiteral("Cannot create object; only enums defined!"));
-    qmlRegisterUncreatableType<RegistrationController>(APPLICATION_ID,
-                                                       1,
-                                                       0,
-                                                       "RegistrationController",
-                                                       QStringLiteral("Cannot create object; only enums defined!"));
+    qmlRegisterUncreatableType<RegistrationController>(APPLICATION_ID, 1, 0, "RegistrationController", QStringLiteral("Not creatable from QML"));
     qmlRegisterUncreatableType<AccountMigrationController>(APPLICATION_ID,
                                                            1,
                                                            0,
                                                            "AccountMigrationController",
                                                            QStringLiteral("Cannot create object; only enums defined!"));
-    qmlRegisterUncreatableType<ChatState>(APPLICATION_ID, 1, 0, "ChatState", QStringLiteral("Cannot create object; only enums defined"));
-    qmlRegisterUncreatableType<RosterModel>(APPLICATION_ID, 1, 0, "RosterModel", QStringLiteral("Cannot create object; only enums defined!"));
     qmlRegisterUncreatableType<HostCompletionModel>(APPLICATION_ID, 1, 0, "HostCompletionModel", QStringLiteral("Cannot create object; only enums defined!"));
-    qmlRegisterUncreatableType<RosterController>(APPLICATION_ID, 1, 0, "RosterController", QStringLiteral("Cannot create object; only enums defined!"));
+    qmlRegisterUncreatableType<RosterController>(APPLICATION_ID, 1, 0, "RosterController", QStringLiteral("Not creatable from QML"));
+    qmlRegisterUncreatableType<Call>(APPLICATION_ID, 1, 0, "Call", QStringLiteral("Not creatable from QML"));
 
     // Q_GADGET
     qmlRegisterUncreatableMetaObject(Emoji::staticMetaObject, APPLICATION_ID, 1, 0, "Emoji", QStringLiteral("Used by emoji models"));
@@ -582,12 +605,7 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
                                      QStringLiteral("Cannot create object; only enums defined!"));
     qmlRegisterUncreatableMetaObject(Message::staticMetaObject, APPLICATION_ID, 1, 0, "Message", QStringLiteral("Cannot create object; only enums defined!"));
     qmlRegisterUncreatableMetaObject(QMimeType::staticMetaObject, APPLICATION_ID, 1, 0, "QMimeType", QStringLiteral("QMimeType type usable"));
-    qmlRegisterUncreatableMetaObject(ChatState::staticMetaObject,
-                                     APPLICATION_ID,
-                                     1,
-                                     0,
-                                     "ChatState",
-                                     QStringLiteral("Can't create object; only enums defined!"));
+    qmlRegisterUncreatableType<ChatStateController>(APPLICATION_ID, 1, 0, "ChatStateController", QStringLiteral("Not creatable from QML"));
 
     // Q_NAMESPACE
     qmlRegisterUncreatableMetaObject(Enums::staticMetaObject, APPLICATION_ID, 1, 0, "Enums", QStringLiteral("Can't create object; only enums defined!"));
@@ -625,13 +643,18 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     });
 
     engine.load(QUrl(QStringLiteral("qrc:/qml/main.qml")));
-    if (engine.rootObjects().isEmpty())
+    if (engine.rootObjects().isEmpty()) {
         return -1;
+    }
 
 #ifdef Q_OS_ANDROID
     QtAndroid::hideSplashScreen();
 #endif
 
     // enter qt main loop
-    return app.exec();
+    auto returnCode = app.exec();
+
+    gst_deinit();
+
+    return returnCode;
 }
