@@ -287,7 +287,7 @@ push_release_branch() {
 
 set_up_stable_branch() {
     # Create, push and configure a new stable branch if none exists.
-    if [ ! $(git ls-remote --exit-code --heads "${remote}" "refs/heads/${stable_branch}") ]
+    if [ ! $(git ls-remote -b --exit-code "${remote}" "refs/heads/${stable_branch}") ]
     then
         git checkout -b "${stable_branch}" "${remote}/${default_branch}"
         git push "${remote}" "${stable_branch}"
@@ -324,18 +324,18 @@ update_release_branch() {
     # Reset the release commit while keeping all other commits created for the release.
     git reset HEAD~1
 
-    git restore "${doap_file}"
-    update_doap
-
     update_changelog_date
     update_appstream_metadata
 
+    git restore "${doap_file}"
+    update_doap
+
     push_release_branch
 }
 
 merge_release_branch() {
     # Merge the release branch into the stable branch if not already done.
-    if [ ! $(git log "${remote}/${stable_branch}" --pretty=oneline --grep="${release_commit_message}") ]
+    if [[ ! $(git log "${remote}/${stable_branch}" --pretty=oneline --grep="${release_commit_message}") ]]
     then
         # Rebase the release branch on the stable branch.
         git checkout "${release_branch}"
@@ -350,6 +350,7 @@ merge_release_branch() {
     fi
 
     # Clean up since the release branch is not needed anymore.
+    git ls-remote -b --exit-code "${remote}" "refs/heads/${release_branch}" && \
     git push -d "${remote}" "${release_branch}"
     git remote prune "${remote}"
     git branch -D "${release_branch}"
@@ -357,7 +358,7 @@ merge_release_branch() {
 
 merge_stable_branch() {
     # Merge the stable branch into the default branch if not already done.
-    if [ ! $(git log "${remote}/${default_branch}" --pretty=oneline --grep="${release_commit_message}") ]
+    if [[ ! $(git log "${remote}/${default_branch}" --pretty=oneline --grep="${release_commit_message}") ]]
     then
         git checkout "${stable_branch}"
         git pull
